@@ -23,12 +23,15 @@ License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 #include "../../src/client/compositor.h"
 #include "../../src/client/connection_thread.h"
 #include "../../src/client/event_queue.h"
+#include "../../src/client/pointer.h"
+#include "../../src/client/seat.h"
 #include "../../src/client/shell.h"
 #include "../../src/client/surface.h"
 #include "../../src/client/registry.h"
 #include "../../src/server/buffer_interface.h"
 #include "../../src/server/compositor_interface.h"
 #include "../../src/server/display.h"
+#include "../../src/server/seat_interface.h"
 #include "../../src/server/shell_interface.h"
 #include "../../src/server/surface_interface.h"
 // Wayland
@@ -40,6 +43,7 @@ class TestWaylandShell : public QObject
 public:
     explicit TestWaylandShell(QObject *parent = nullptr);
 private Q_SLOTS:
+    void initTestCase();
     void init();
     void cleanup();
 
@@ -47,19 +51,27 @@ private Q_SLOTS:
     void testFullscreen();
     void testMaximize();
     void testToplevel();
+    void testTransient_data();
+    void testTransient();
     void testPing();
     void testTitle();
     void testWindowClass();
     void testDestroy();
     void testCast();
+    void testMove();
+    void testResize_data();
+    void testResize();
 
 private:
     KWayland::Server::Display *m_display;
     KWayland::Server::CompositorInterface *m_compositorInterface;
     KWayland::Server::ShellInterface *m_shellInterface;
+    KWayland::Server::SeatInterface *m_seatInterface;
     KWayland::Client::ConnectionThread *m_connection;
     KWayland::Client::Compositor *m_compositor;
     KWayland::Client::Shell *m_shell;
+    KWayland::Client::Seat *m_seat;
+    KWayland::Client::Pointer *m_pointer;
     KWayland::Client::EventQueue *m_queue;
     QThread *m_thread;
 };
@@ -71,14 +83,22 @@ TestWaylandShell::TestWaylandShell(QObject *parent)
     , m_display(nullptr)
     , m_compositorInterface(nullptr)
     , m_shellInterface(nullptr)
+    , m_seatInterface(nullptr)
     , m_connection(nullptr)
     , m_compositor(nullptr)
     , m_shell(nullptr)
+    , m_seat(nullptr)
+    , m_pointer(nullptr)
     , m_queue(nullptr)
     , m_thread(nullptr)
 {
 }
 
+void TestWaylandShell::initTestCase()
+{
+    qRegisterMetaType<Qt::Edges>();
+}
+
 void TestWaylandShell::init()
 {
     using namespace KWayland::Server;
@@ -98,6 +118,12 @@ void TestWaylandShell::init()
     m_shellInterface->create();
     QVERIFY(m_shellInterface->isValid());
 
+    m_seatInterface = m_display->createSeat(m_display);
+    QVERIFY(m_seatInterface);
+    m_seatInterface->setHasPointer(true);
+    m_seatInterface->create();
+    QVERIFY(m_seatInterface->isValid());
+
     // setup connection
     m_connection = new KWayland::Client::ConnectionThread;
     QSignalSpy connectedSpy(m_connection, SIGNAL(connected()));
@@ -115,16 +141,21 @@ void TestWaylandShell::init()
     m_queue->setup(m_connection);
     QVERIFY(m_queue->isValid());
 
+    using namespace KWayland::Client;
     KWayland::Client::Registry registry;
+    QSignalSpy interfacesAnnouncedSpy(&registry, &Registry::interfacesAnnounced);
+    QVERIFY(interfacesAnnouncedSpy.isValid());
     QSignalSpy compositorSpy(&registry, SIGNAL(compositorAnnounced(quint32,quint32)));
     QSignalSpy shellSpy(&registry, SIGNAL(shellAnnounced(quint32,quint32)));
+    QSignalSpy seatAnnouncedSpy(&registry, &Registry::seatAnnounced);
+    QVERIFY(seatAnnouncedSpy.isValid());
     QVERIFY(!registry.eventQueue());
     registry.setEventQueue(m_queue);
     QCOMPARE(registry.eventQueue(), m_queue);
     registry.create(m_connection->display());
     QVERIFY(registry.isValid());
     registry.setup();
-    QVERIFY(compositorSpy.wait());
+    QVERIFY(interfacesAnnouncedSpy.wait());
 
     m_compositor = new KWayland::Client::Compositor(this);
     m_compositor->setup(registry.bindCompositor(compositorSpy.first().first().value<quint32>(), compositorSpy.first().last().value<quint32>()));
@@ -135,10 +166,28 @@ void TestWaylandShell::init()
     }
     m_shell = registry.createShell(shellSpy.first().first().value<quint32>(), shellSpy.first().last().value<quint32>(), this);
     QVERIFY(m_shell->isValid());
+
+    QVERIFY(!seatAnnouncedSpy.isEmpty());
+    m_seat = registry.createSeat(seatAnnouncedSpy.first().first().value<quint32>(), seatAnnouncedSpy.first().last().value<quint32>(), this);
+    QVERIFY(seatAnnouncedSpy.isValid());
+    QSignalSpy hasPointerSpy(m_seat, &Seat::hasPointerChanged);
+    QVERIFY(hasPointerSpy.isValid());
+    QVERIFY(hasPointerSpy.wait());
+    QVERIFY(hasPointerSpy.first().first().toBool());
+    m_pointer = m_seat->createPointer(m_seat);
+    QVERIFY(m_pointer->isValid());
 }
 
 void TestWaylandShell::cleanup()
 {
+    if (m_pointer) {
+        delete m_pointer;
+        m_pointer = nullptr;
+    }
+    if (m_seat) {
+        delete m_seat;
+        m_seat = nullptr;
+    }
     if (m_shell) {
         delete m_shell;
         m_shell = nullptr;
@@ -162,6 +211,9 @@ void TestWaylandShell::cleanup()
         m_thread = nullptr;
     }
 
+    delete m_seatInterface;
+    m_seatInterface = nullptr;
+
     delete m_shellInterface;
     m_shellInterface = nullptr;
 
@@ -334,6 +386,79 @@ void TestWaylandShell::testToplevel()
     QVERIFY(!toplevelSpy.first().first().toBool());
 }
 
+void TestWaylandShell::testTransient_data()
+{
+    QTest::addColumn<bool>("keyboardFocus");
+
+    QTest::newRow("focus") << true;
+    QTest::newRow("no focus") << false;
+}
+
+void TestWaylandShell::testTransient()
+{
+    using namespace KWayland::Server;
+    using namespace KWayland::Client;
+    QScopedPointer<Surface> s(m_compositor->createSurface());
+    QVERIFY(!s.isNull());
+    QVERIFY(s->isValid());
+    ShellSurface *surface = m_shell->createSurface(s.data(), m_shell);
+
+    QSignalSpy serverSurfaceSpy(m_shellInterface, &ShellInterface::surfaceCreated);
+    QVERIFY(serverSurfaceSpy.isValid());
+    QVERIFY(serverSurfaceSpy.wait());
+    ShellSurfaceInterface *serverSurface = serverSurfaceSpy.first().first().value<ShellSurfaceInterface*>();
+    QVERIFY(serverSurface);
+    QSignalSpy acceptsKeyboardFocusChangedSpy(serverSurface, &ShellSurfaceInterface::acceptsKeyboardFocusChanged);
+    QVERIFY(acceptsKeyboardFocusChangedSpy.isValid());
+    QCOMPARE(serverSurface->isToplevel(), true);
+    QCOMPARE(serverSurface->isPopup(), false);
+    QCOMPARE(serverSurface->isTransient(), false);
+    QCOMPARE(serverSurface->transientFor(), QPointer<SurfaceInterface>());
+    QCOMPARE(serverSurface->transientOffset(), QPoint());
+    QVERIFY(serverSurface->acceptsKeyboardFocus());
+    QVERIFY(acceptsKeyboardFocusChangedSpy.isEmpty());
+
+    QSignalSpy transientSpy(serverSurface, &ShellSurfaceInterface::transientChanged);
+    QVERIFY(transientSpy.isValid());
+    QSignalSpy transientOffsetSpy(serverSurface, &ShellSurfaceInterface::transientOffsetChanged);
+    QVERIFY(transientOffsetSpy.isValid());
+    QSignalSpy transientForChangedSpy(serverSurface, &ShellSurfaceInterface::transientForChanged);
+    QVERIFY(transientForChangedSpy.isValid());
+
+    QScopedPointer<Surface> s2(m_compositor->createSurface());
+    m_shell->createSurface(s2.data(), m_shell);
+    serverSurfaceSpy.clear();
+    QVERIFY(serverSurfaceSpy.wait());
+    ShellSurfaceInterface *serverSurface2 = serverSurfaceSpy.first().first().value<ShellSurfaceInterface*>();
+    QVERIFY(serverSurface2 != serverSurface);
+    QVERIFY(serverSurface2);
+    QVERIFY(serverSurface2->acceptsKeyboardFocus());
+
+    QFETCH(bool, keyboardFocus);
+    surface->setTransient(s2.data(), QPoint(10, 20), keyboardFocus ? ShellSurface::TransientFlag::Default : ShellSurface::TransientFlag::NoFocus);
+    QVERIFY(transientSpy.wait());
+    QCOMPARE(transientSpy.count(), 1);
+    QCOMPARE(transientSpy.first().first().toBool(), true);
+    QCOMPARE(transientOffsetSpy.count(), 1);
+    QCOMPARE(transientOffsetSpy.first().first().toPoint(), QPoint(10, 20));
+    QCOMPARE(transientForChangedSpy.count(), 1);
+    QCOMPARE(serverSurface->isToplevel(), true);
+    QCOMPARE(serverSurface->isPopup(), false);
+    QCOMPARE(serverSurface->isTransient(), true);
+    QCOMPARE(serverSurface->transientFor(), QPointer<SurfaceInterface>(serverSurface2->surface()));
+    QCOMPARE(serverSurface->transientOffset(), QPoint(10, 20));
+    QCOMPARE(serverSurface->acceptsKeyboardFocus(), keyboardFocus);
+    QCOMPARE(acceptsKeyboardFocusChangedSpy.isEmpty(), keyboardFocus);
+    QCOMPARE(acceptsKeyboardFocusChangedSpy.count(), keyboardFocus ? 0 : 1);
+
+    QCOMPARE(serverSurface2->isToplevel(), true);
+    QCOMPARE(serverSurface2->isPopup(), false);
+    QCOMPARE(serverSurface2->isTransient(), false);
+    QCOMPARE(serverSurface2->transientFor(), QPointer<SurfaceInterface>());
+    QCOMPARE(serverSurface2->transientOffset(), QPoint());
+    QVERIFY(serverSurface2->acceptsKeyboardFocus());
+}
+
 void TestWaylandShell::testPing()
 {
     using namespace KWayland::Server;
@@ -437,6 +562,8 @@ void TestWaylandShell::testDestroy()
     QVERIFY(surface->isValid());
 
     connect(m_connection, &ConnectionThread::connectionDied, m_shell, &Shell::destroy);
+    connect(m_connection, &ConnectionThread::connectionDied, m_pointer, &Pointer::destroy);
+    connect(m_connection, &ConnectionThread::connectionDied, m_seat, &Seat::destroy);
     connect(m_connection, &ConnectionThread::connectionDied, m_compositor, &Compositor::destroy);
     connect(m_connection, &ConnectionThread::connectionDied, s.data(), &Surface::destroy);
     connect(m_connection, &ConnectionThread::connectionDied, m_queue, &EventQueue::destroy);
@@ -447,6 +574,7 @@ void TestWaylandShell::testDestroy()
     m_display = nullptr;
     m_compositorInterface = nullptr;
     m_shellInterface = nullptr;
+    m_seatInterface = nullptr;
     QVERIFY(connectionDiedSpy.wait());
 
     QVERIFY(!m_shell->isValid());
@@ -480,5 +608,96 @@ void TestWaylandShell::testCast()
     QCOMPARE((wl_shell*)s2, wlShell);
 }
 
+void TestWaylandShell::testMove()
+{
+    using namespace KWayland::Client;
+    using namespace KWayland::Server;
+    QScopedPointer<KWayland::Client::Surface> s(m_compositor->createSurface());
+    QVERIFY(!s.isNull());
+    QVERIFY(s->isValid());
+    ShellSurface *surface = m_shell->createSurface(s.data(), m_shell);
+
+    QSignalSpy serverSurfaceSpy(m_shellInterface, &ShellInterface::surfaceCreated);
+    QVERIFY(serverSurfaceSpy.isValid());
+    QVERIFY(serverSurfaceSpy.wait());
+    ShellSurfaceInterface *serverSurface = serverSurfaceSpy.first().first().value<ShellSurfaceInterface*>();
+    QVERIFY(serverSurface);
+    QSignalSpy moveRequestedSpy(serverSurface, &ShellSurfaceInterface::moveRequested);
+    QVERIFY(moveRequestedSpy.isValid());
+
+    QSignalSpy pointerButtonChangedSpy(m_pointer, &Pointer::buttonStateChanged);
+    QVERIFY(pointerButtonChangedSpy.isValid());
+
+    m_seatInterface->setFocusedPointerSurface(serverSurface->surface());
+    m_seatInterface->pointerButtonPressed(Qt::LeftButton);
+    QVERIFY(pointerButtonChangedSpy.wait());
+
+    surface->requestMove(m_seat, pointerButtonChangedSpy.first().first().value<quint32>());
+    QVERIFY(moveRequestedSpy.wait());
+    QCOMPARE(moveRequestedSpy.count(), 1);
+    QCOMPARE(moveRequestedSpy.first().at(0).value<SeatInterface*>(), m_seatInterface);
+    QCOMPARE(moveRequestedSpy.first().at(1).value<quint32>(), m_seatInterface->pointerButtonSerial(Qt::LeftButton));
+}
+
+void TestWaylandShell::testResize_data()
+{
+    QTest::addColumn<Qt::Edges>("resizeEdge");
+    QTest::addColumn<Qt::Edges>("expectedEdge");
+
+    QTest::newRow("None") << Qt::Edges() << Qt::Edges();
+
+    QTest::newRow("Top")    << Qt::Edges(Qt::TopEdge)    << Qt::Edges(Qt::TopEdge);
+    QTest::newRow("Bottom") << Qt::Edges(Qt::BottomEdge) << Qt::Edges(Qt::BottomEdge);
+    QTest::newRow("Left")   << Qt::Edges(Qt::LeftEdge)   << Qt::Edges(Qt::LeftEdge);
+    QTest::newRow("Right")  << Qt::Edges(Qt::RightEdge)  << Qt::Edges(Qt::RightEdge);
+
+    QTest::newRow("Top Left")     << Qt::Edges(Qt::TopEdge | Qt::LeftEdge)     << Qt::Edges(Qt::TopEdge | Qt::LeftEdge);
+    QTest::newRow("Top Right")    << Qt::Edges(Qt::TopEdge | Qt::RightEdge)    << Qt::Edges(Qt::TopEdge | Qt::RightEdge);
+    QTest::newRow("Bottom Left")  << Qt::Edges(Qt::BottomEdge | Qt::LeftEdge)  << Qt::Edges(Qt::BottomEdge | Qt::LeftEdge);
+    QTest::newRow("Bottom Right") << Qt::Edges(Qt::BottomEdge | Qt::RightEdge) << Qt::Edges(Qt::BottomEdge | Qt::RightEdge);
+
+    // invalid combinations
+    QTest::newRow("Top Bottom") << Qt::Edges(Qt::TopEdge | Qt::BottomEdge) << Qt::Edges();
+    QTest::newRow("Left Right") << Qt::Edges(Qt::RightEdge | Qt::LeftEdge) << Qt::Edges();
+    QTest::newRow("Top Bottom Right")  << Qt::Edges(Qt::TopEdge | Qt::BottomEdge | Qt::RightEdge)  << Qt::Edges();
+    QTest::newRow("Top Bottom Left")   << Qt::Edges(Qt::TopEdge | Qt::BottomEdge | Qt::LeftEdge)   << Qt::Edges();
+    QTest::newRow("Left Right Top")    << Qt::Edges(Qt::RightEdge | Qt::LeftEdge | Qt::TopEdge)    << Qt::Edges();
+    QTest::newRow("Left Right Bottom") << Qt::Edges(Qt::RightEdge | Qt::LeftEdge | Qt::BottomEdge) << Qt::Edges();
+    QTest::newRow("All") << Qt::Edges(Qt::RightEdge | Qt::LeftEdge | Qt::BottomEdge | Qt::TopEdge) << Qt::Edges();
+}
+
+void TestWaylandShell::testResize()
+{
+    using namespace KWayland::Client;
+    using namespace KWayland::Server;
+    QScopedPointer<KWayland::Client::Surface> s(m_compositor->createSurface());
+    QVERIFY(!s.isNull());
+    QVERIFY(s->isValid());
+    ShellSurface *surface = m_shell->createSurface(s.data(), m_shell);
+
+    QSignalSpy serverSurfaceSpy(m_shellInterface, &ShellInterface::surfaceCreated);
+    QVERIFY(serverSurfaceSpy.isValid());
+    QVERIFY(serverSurfaceSpy.wait());
+    ShellSurfaceInterface *serverSurface = serverSurfaceSpy.first().first().value<ShellSurfaceInterface*>();
+    QVERIFY(serverSurface);
+    QSignalSpy resizeRequestedSpy(serverSurface, &ShellSurfaceInterface::resizeRequested);
+    QVERIFY(resizeRequestedSpy.isValid());
+
+    QSignalSpy pointerButtonChangedSpy(m_pointer, &Pointer::buttonStateChanged);
+    QVERIFY(pointerButtonChangedSpy.isValid());
+
+    m_seatInterface->setFocusedPointerSurface(serverSurface->surface());
+    m_seatInterface->pointerButtonPressed(Qt::LeftButton);
+    QVERIFY(pointerButtonChangedSpy.wait());
+
+    QFETCH(Qt::Edges, resizeEdge);
+    surface->requestResize(m_seat, pointerButtonChangedSpy.first().first().value<quint32>(), resizeEdge);
+    QVERIFY(resizeRequestedSpy.wait());
+    QCOMPARE(resizeRequestedSpy.count(), 1);
+    QCOMPARE(resizeRequestedSpy.first().at(0).value<SeatInterface*>(), m_seatInterface);
+    QCOMPARE(resizeRequestedSpy.first().at(1).value<quint32>(), m_seatInterface->pointerButtonSerial(Qt::LeftButton));
+    QTEST(resizeRequestedSpy.first().at(2).value<Qt::Edges>(), "expectedEdge");
+}
+
 QTEST_GUILESS_MAIN(TestWaylandShell)
 #include "test_wayland_shell.moc"
