@@ -33,22 +33,47 @@ struct wl_event_loop;
 
 namespace KWayland
 {
+/**
+ * @short KWayland Server.
+ *
+ * This namespace groups all classes related to the Server module.
+ *
+ * The main entry point into the KWayland::Server API is the Display class.
+ * It allows to create a Wayland server and create various global objects on it.
+ *
+ * KWayland::Server is an API to easily create a head-less Wayland server with a
+ * Qt style API.
+ *
+ * @see Display
+ **/
 namespace Server
 {
 
 class CompositorInterface;
 class DataDeviceManagerInterface;
+class DpmsManagerInterface;
 class IdleInterface;
 class FakeInputInterface;
 class OutputInterface;
+class OutputDeviceInterface;
+class OutputConfigurationInterface;
+class OutputManagementInterface;
 class PlasmaShellInterface;
 class PlasmaWindowManagementInterface;
 class QtSurfaceExtensionInterface;
 class SeatInterface;
 class ShadowManagerInterface;
+class BlurManagerInterface;
+class ContrastManagerInterface;
+class SlideManagerInterface;
 class ShellInterface;
 class SubCompositorInterface;
 
+/**
+ * @brief Class holding the Wayland server display loop.
+ *
+ * @todo Improve documentation
+ **/
 class KWAYLANDSERVER_EXPORT Display : public QObject
 {
     Q_OBJECT
@@ -114,18 +139,27 @@ public:
     void removeOutput(OutputInterface *output);
     QList<OutputInterface*> outputs() const;
 
+    OutputDeviceInterface *createOutputDevice(QObject *parent = nullptr);
+    void removeOutputDevice(OutputDeviceInterface *output);
+    QList<OutputDeviceInterface*> outputDevices() const;
+
     CompositorInterface *createCompositor(QObject *parent = nullptr);
     void createShm();
     ShellInterface *createShell(QObject *parent = nullptr);
     SeatInterface *createSeat(QObject *parent = nullptr);
     SubCompositorInterface *createSubCompositor(QObject *parent = nullptr);
     DataDeviceManagerInterface *createDataDeviceManager(QObject *parent = nullptr);
+    OutputManagementInterface *createOutputManagement(QObject *parent = nullptr);
     PlasmaShellInterface *createPlasmaShell(QObject *parent = nullptr);
     PlasmaWindowManagementInterface *createPlasmaWindowManagement(QObject *parent = nullptr);
     QtSurfaceExtensionInterface *createQtSurfaceExtension(QObject *parent = nullptr);
     IdleInterface *createIdle(QObject *parent = nullptr);
     FakeInputInterface *createFakeInput(QObject *parent = nullptr);
     ShadowManagerInterface *createShadowManager(QObject *parent = nullptr);
+    BlurManagerInterface *createBlurManager(QObject *parent = nullptr);
+    ContrastManagerInterface *createContrastManager(QObject *parent = nullptr);
+    SlideManagerInterface *createSlideManager(QObject *parent = nullptr);
+    DpmsManagerInterface *createDpmsManager(QObject *parent = nullptr);
 
     /**
      * Gets the ClientConnection for the given @p client.
