@@ -36,6 +36,7 @@ namespace Server
 
 class Display;
 class PlasmaWindowInterface;
+class SurfaceInterface;
 
 class KWAYLANDSERVER_EXPORT PlasmaWindowManagementInterface : public Global
 {
@@ -82,10 +83,18 @@ public:
     void setMinimizeable(bool set);
     void setMaximizeable(bool set);
     void setFullscreenable(bool set);
+    void setSkipTaskbar(bool skip);
     void setThemedIconName(const QString &iconName);
 
     void unmap();
 
+    /**
+     * @returns Geometries of the taskbar entries, indicized by the
+     *          surface of the panels
+     * @since 5.5
+     */
+    QHash<SurfaceInterface*, QRect> minimizedGeometries() const;
+
 Q_SIGNALS:
     void closeRequested();
     void virtualDesktopRequested(quint32 desktop);
@@ -100,6 +109,8 @@ Q_SIGNALS:
     void minimizeableRequested(bool set);
     void maximizeableRequested(bool set);
     void fullscreenableRequested(bool set);
+    void skipTaskbarRequested(bool set);
+    QRect minimizedGeometriesChanged();
 
 private:
     friend class PlasmaWindowManagementInterface;
